import { Host, h } from "@stencil/core";
import { checkResizeObserver, DEVICE_SIZES, getCurrentDeviceSize, } from "../../utils/helpers";
// Added ResizeObserver to find out width of breadcrumbs and parents. Use side navigation long title for ref.
export class BreadcrumbGroup {
    constructor() {
        this.ADD_CLASS_DELAY = 50;
        this.IC_BREADCRUMB = "ic-breadcrumb";
        this.resizeObserver = null;
        this.SHOW_BACK_ICON = "show-back-icon";
        this.deviceSize = DEVICE_SIZES.XL;
        this.expandedBreadcrumbs = false;
        /**
         * If `true`, display only a single breadcrumb for the parent page with a back icon.
         */
        this.backBreadcrumbOnly = false;
        /**
         * If `true`, all breadcrumbs between the first and last breadcrumb will be collapsed.
         */
        this.collapsed = false;
        /**
         * If `true`, the breadcrumb group will display as black in the light theme, and white in the dark theme.
         */
        this.monochrome = false;
        /**
         * Sets the theme color to the dark or light theme color. "inherit" will set the color based on the system settings or ic-theme component.
         */
        this.theme = "inherit";
        this.setBreadcrumbTheme = () => {
            const allBreadcrumbs = Array.from(this.el.querySelectorAll(this.IC_BREADCRUMB));
            allBreadcrumbs.forEach((breadcrumb) => {
                breadcrumb.theme = this.theme;
            });
        };
        this.setBreadcrumbMonochrome = () => {
            const allBreadcrumbs = Array.from(this.el.querySelectorAll(this.IC_BREADCRUMB));
            allBreadcrumbs.forEach((breadcrumb) => {
                breadcrumb.monochrome = this.monochrome;
            });
        };
        this.setBackBreadcrumb = () => {
            if (this.backBreadcrumbOnly) {
                this.setLastParentCollapsedBackBreadcrumb();
            }
            else {
                this.revertLastParentCollapsedBreadcrumb();
            }
        };
        this.setBackBreadcrumbAttr = () => {
            if (this.lastParentBreadcrumb) {
                this.lastParentBreadcrumb.classList.add("show");
                this.lastParentBreadcrumb.setAttribute(this.SHOW_BACK_ICON, "true");
            }
        };
        this.getLastParentBreadcrumb = () => {
            const allBreadcrumbs = Array.from(this.el.querySelectorAll(this.IC_BREADCRUMB));
            if (allBreadcrumbs.length === 1) {
                return null;
            }
            this.breadcrumbs = allBreadcrumbs;
            this.breadcrumb = this.breadcrumbs[this.breadcrumbs.length - 2];
            return this.breadcrumb;
        };
        this.lastParentBreadcrumb = this.getLastParentBreadcrumb();
        this.setDefaultBreadcrumbs = () => {
            const allBreadcrumbs = Array.from(this.el.querySelectorAll(this.IC_BREADCRUMB));
            allBreadcrumbs.forEach((breadcrumb) => {
                breadcrumb.setAttribute(this.SHOW_BACK_ICON, "false");
            });
        };
        this.setCollapsed = () => {
            const allBreadcrumbs = Array.from(this.el.querySelectorAll(this.IC_BREADCRUMB));
            const firstBreadcrumb = allBreadcrumbs[0];
            if (this.collapsedBreadcrumbs) {
                this.collapsedBreadcrumbs.forEach((breadcrumb) => {
                    breadcrumb.classList.remove("visuallyhidden");
                    breadcrumb.classList.remove("fade");
                });
            }
            if (this.collapsed) {
                this.renderCollapsedBreadcrumb();
                this.collapsedBreadcrumbs = allBreadcrumbs
                    .splice(1, allBreadcrumbs.length - 2)
                    .filter((breadcrumb) => !breadcrumb.classList.contains("collapsed-breadcrumb-wrapper"));
                if (!this.backBreadcrumbOnly) {
                    this.collapsedBreadcrumbs.forEach((breadcrumb) => breadcrumb.classList.add("hide"));
                }
                if (firstBreadcrumb) {
                    firstBreadcrumb.insertAdjacentElement("afterend", this.collapsedBreadcrumbWrapper);
                }
            }
            else {
                this.collapsedBreadcrumbWrapper.remove();
            }
        };
        this.clickHandler = () => {
            this.handleHiddenCollapsedBreadcrumbs();
        };
        this.renderCollapsedBreadcrumb = () => {
            if (this.collapsedBreadcrumbEl === undefined) {
                this.collapsedBreadcrumbWrapper = document.createElement("ic-breadcrumb");
                this.collapsedBreadcrumbWrapper.classList.add("collapsed-breadcrumb-wrapper");
                this.collapsedBreadcrumbEl = document.createElement("button");
                const ariaLabel = document.createElement("span");
                ariaLabel.id = "collapsed-button-label";
                ariaLabel.innerText = "Collapsed breadcrumbs";
                ariaLabel.className = "hide";
                this.collapsedBreadcrumbEl.setAttribute("aria-labelledby", "collapsed-button-label");
                const ariaDescribed = document.createElement("span");
                ariaDescribed.id = "collapsed-button-described";
                ariaDescribed.innerText = "Select to view collapsed breadcrumbs";
                ariaDescribed.className = "hide";
                this.collapsedBreadcrumbEl.setAttribute("aria-describedby", "collapsed-button-described");
                this.collapsedBreadcrumbEl.id = "collapsed-ellipsis";
                this.collapsedBreadcrumbEl.innerText = "...";
                this.collapsedBreadcrumbEl.classList.add("collapsed-breadcrumb");
                this.collapsedBreadcrumbEl.addEventListener("click", this.clickHandler);
                this.collapsedBreadcrumbWrapper.append(ariaDescribed);
                this.collapsedBreadcrumbWrapper.append(ariaLabel);
                this.collapsedBreadcrumbWrapper.append(this.collapsedBreadcrumbEl);
            }
        };
        this.handleHiddenCollapsedBreadcrumbs = () => {
            this.collapsedBreadcrumbWrapper.remove();
            this.collapsedBreadcrumbs.forEach((breadcrumb) => {
                breadcrumb.classList.add("visuallyhidden");
                breadcrumb.classList.remove("hide");
                setTimeout(() => {
                    breadcrumb.classList.add("fade");
                }, this.ADD_CLASS_DELAY);
                this.removeVisuallyHiddenClass(breadcrumb);
            });
            this.expandedBreadcrumbs = true;
            // Set focus to first unhidden breadcrumb
            this.collapsedBreadcrumbs[0].setFocus();
        };
        this.transitionendHandler = (event) => {
            if (event.propertyName === "opacity") {
                event.target.classList.remove("visuallyhidden");
            }
        };
        this.removeVisuallyHiddenClass = (breadcrumb) => {
            breadcrumb.addEventListener("transitionend", this.transitionendHandler);
        };
        this.setLastParentCollapsedBackBreadcrumb = () => {
            this.lastParentBreadcrumb = this.getLastParentBreadcrumb();
            this.setBackBreadcrumbAttr();
            if (this.lastParentBreadcrumb) {
                this.lastParentBreadcrumb.classList.remove("hide");
                this.lastParentBreadcrumb.classList.add("show");
            }
        };
        this.revertLastParentCollapsedBreadcrumb = () => {
            if (this.lastParentBreadcrumb) {
                this.lastParentBreadcrumb.setAttribute(this.SHOW_BACK_ICON, "false");
                if (this.collapsed) {
                    this.lastParentBreadcrumb.classList.add("hide");
                }
            }
        };
        this.resizeObserverCallback = (currSize) => {
            if (currSize !== this.deviceSize) {
                this.deviceSize = currSize;
                if (this.deviceSize <= DEVICE_SIZES.S) {
                    this.el.setAttribute("back-breadcrumb-only", "true");
                    if (this.collapsed) {
                        this.setLastParentCollapsedBackBreadcrumb();
                    }
                    else {
                        this.setBackBreadcrumb();
                    }
                }
                else {
                    this.el.setAttribute("back-breadcrumb-only", "false");
                    if (this.collapsed && this.breadcrumbs && this.breadcrumbs.length > 2) {
                        this.revertLastParentCollapsedBreadcrumb();
                        if (this.expandedBreadcrumbs) {
                            this.setDefaultBreadcrumbs();
                        }
                        else {
                            this.setCollapsed();
                        }
                    }
                    else {
                        this.setDefaultBreadcrumbs();
                    }
                }
            }
        };
        this.runResizeObserver = () => {
            this.resizeObserver = new ResizeObserver(() => {
                const currSize = getCurrentDeviceSize();
                this.resizeObserverCallback(currSize);
            });
            this.resizeObserver.observe(this.el);
        };
    }
    watchBackBreadcrumbHandler() {
        this.setBackBreadcrumb();
    }
    watchCollapsedHandler() {
        this.setCollapsed();
    }
    watchMonochromeHandler() {
        this.setBreadcrumbMonochrome();
    }
    watchThemeHandler() {
        this.setBreadcrumbTheme();
    }
    componentWillLoad() {
        const allBreadcrumbs = Array.from(this.el.querySelectorAll(this.IC_BREADCRUMB));
        this.setBreadcrumbTheme();
        this.setBreadcrumbMonochrome();
        if (this.backBreadcrumbOnly) {
            this.setBackBreadcrumb();
        }
        else {
            checkResizeObserver(this.runResizeObserver);
        }
        if (this.collapsed && allBreadcrumbs.length > 2) {
            if (getCurrentDeviceSize() === DEVICE_SIZES.S) {
                this.setLastParentCollapsedBackBreadcrumb();
            }
            else {
                this.setCollapsed();
            }
        }
    }
    disconnectedCallback() {
        this.breadcrumb &&
            this.breadcrumb.removeEventListener("transitionend", this.transitionendHandler);
        this.collapsedBreadcrumbEl &&
            this.collapsedBreadcrumbEl.removeEventListener("click", this.clickHandler);
    }
    render() {
        return (h(Host, { key: 'b505c4bef9ebc23c7f77b21f77f9db5975d0eea1', class: {
                "ic-breadcrumb-group-back": !!this.backBreadcrumbOnly,
                "ic-breadcrumb-group-collapsed": !!this.collapsed,
                [`ic-theme-${this.theme}`]: this.theme !== "inherit",
            } }, h("nav", { key: 'ec1db09900bbff5be1e7f2e83518834a0eb0ff22', "aria-label": "breadcrumbs" }, h("ol", { key: 'b13e6daeefb0e4d1785221249f4d38bf3f207922' }, h("slot", { key: 'b33ff6c552a6f7251afeb6a804b4761e0febe198' })))));
    }
    static get is() { return "ic-breadcrumb-group"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() {
        return {
            "$": ["ic-breadcrumb-group.css"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["ic-breadcrumb-group.css"]
        };
    }
    static get properties() {
        return {
            "backBreadcrumbOnly": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "If `true`, display only a single breadcrumb for the parent page with a back icon."
                },
                "getter": false,
                "setter": false,
                "attribute": "back-breadcrumb-only",
                "reflect": false,
                "defaultValue": "false"
            },
            "collapsed": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "If `true`, all breadcrumbs between the first and last breadcrumb will be collapsed."
                },
                "getter": false,
                "setter": false,
                "attribute": "collapsed",
                "reflect": false,
                "defaultValue": "false"
            },
            "monochrome": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "If `true`, the breadcrumb group will display as black in the light theme, and white in the dark theme."
                },
                "getter": false,
                "setter": false,
                "attribute": "monochrome",
                "reflect": false,
                "defaultValue": "false"
            },
            "theme": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "IcThemeMode",
                    "resolved": "\"dark\" | \"inherit\" | \"light\" | undefined",
                    "references": {
                        "IcThemeMode": {
                            "location": "import",
                            "path": "../../utils/types",
                            "id": "src/utils/types.ts::IcThemeMode"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Sets the theme color to the dark or light theme color. \"inherit\" will set the color based on the system settings or ic-theme component."
                },
                "getter": false,
                "setter": false,
                "attribute": "theme",
                "reflect": false,
                "defaultValue": "\"inherit\""
            }
        };
    }
    static get states() {
        return {
            "deviceSize": {},
            "expandedBreadcrumbs": {}
        };
    }
    static get elementRef() { return "el"; }
    static get watchers() {
        return [{
                "propName": "backBreadcrumbOnly",
                "methodName": "watchBackBreadcrumbHandler"
            }, {
                "propName": "collapsed",
                "methodName": "watchCollapsedHandler"
            }, {
                "propName": "monochrome",
                "methodName": "watchMonochromeHandler"
            }, {
                "propName": "theme",
                "methodName": "watchThemeHandler"
            }];
    }
}
//# sourceMappingURL=ic-breadcrumb-group.js.map
