import { h, Host, } from "@stencil/core";
import { onComponentRequiredPropUndefined, isSlotUsed, getBrandFromContext, removeDisabledFalse, renderDynamicChildSlots, } from "../../utils/helpers";
import { IcBrandForegroundEnum, } from "../../utils/types";
import chevronIcon from "../../assets/chevron-icon.svg";
/**
 * @slot heading - Content will be placed at the top of the card to the right of the icon.
 * @slot message - Content will be placed in the main body of the card.
 * @slot subheading - Content will be placed below the card heading.
 * @slot adornment - Content will be placed below the card subheading.
 * @slot image-top - Content will be placed at the top of the card above all other content.
 * @slot image-mid - Content will be placed below the card heading section.
 * @slot icon - Content will be placed to the left of the card heading.
 * @slot interaction-button - Content will be placed in the top right corner of the heading section.
 * @slot interaction-controls - Content will be placed below the card message.
 * @slot expanded-content - Content will be placed below the interaction controls but will not initially be rendered.
 * @slot badge - Badge component overlaying the top right of the card.
 */
export class CardVertical {
    constructor() {
        this.hostMutationObserver = null;
        this.areaExpanded = false;
        this.isFocussed = false;
        this.monochrome = false;
        this.parentEl = null;
        this.parentIsAnchorTag = false;
        /**
         * If `true`, the card will be a clickable variant, instead of static.
         */
        this.clickable = false;
        /**
         * If `true`, the card will be disabled if it is clickable.
         */
        this.disabled = false;
        /**
         *  If `true`, the card will have an expandable area and expansion toggle button.
         */
        this.expandable = false;
        /**
         *  If `true`, the card will fill the width of the container.
         */
        this.fullWidth = false;
        /**
         * The human language of the linked URL.
         */
        this.hreflang = "";
        /**
         * The main body message of the card.
         */
        this.message = "";
        /**
         * Sets the theme color to the dark or light theme color. "inherit" will set the color based on the system settings or ic-theme component.
         */
        this.theme = "inherit";
        this.parentFocussed = () => {
            this.isFocussed = true;
        };
        this.parentBlurred = () => {
            this.isFocussed = false;
        };
        this.toggleExpanded = () => {
            this.areaExpanded = !this.areaExpanded;
        };
    }
    watchDisabledHandler() {
        removeDisabledFalse(this.disabled, this.el);
    }
    disconnectedCallback() {
        var _a;
        if (this.parentEl && this.parentIsAnchorTag) {
            this.parentEl.removeEventListener("focus", this.parentFocussed);
            this.parentEl.removeEventListener("blur", this.parentBlurred);
        }
        (_a = this.hostMutationObserver) === null || _a === void 0 ? void 0 : _a.disconnect();
    }
    componentWillLoad() {
        var _a;
        this.parentEl = this.el.parentElement;
        if (((_a = this.parentEl) === null || _a === void 0 ? void 0 : _a.tagName) === "A") {
            this.clickable = true;
            this.parentIsAnchorTag = true;
            this.parentEl.classList.add("ic-card-wrapper-link");
            this.parentEl.addEventListener("focus", this.parentFocussed);
            this.parentEl.addEventListener("blur", this.parentBlurred);
        }
        removeDisabledFalse(this.disabled, this.el);
    }
    componentDidLoad() {
        !isSlotUsed(this.el, "heading") &&
            onComponentRequiredPropUndefined([{ prop: this.heading, propName: "heading" }], "Card");
        this.updateTheme();
        this.hostMutationObserver = new MutationObserver((mutationList) => renderDynamicChildSlots(mutationList, [
            "message",
            "adornment",
            "expanded-content",
            "image-top",
            "image-mid",
            "icon",
            "interaction-button",
            "badge",
            "interaction-controls",
        ], this));
        this.hostMutationObserver.observe(this.el, {
            childList: true,
        });
    }
    handleHostClick(event) {
        if (this.disabled) {
            event.stopImmediatePropagation();
        }
    }
    brandChangeHandler(ev) {
        this.updateTheme(ev.detail.mode);
    }
    /**
     * Sets focus on the card.
     */
    async setFocus() {
        var _a, _b;
        const shadowRoot = this.el.shadowRoot;
        if (shadowRoot) {
            if (shadowRoot.querySelector("a")) {
                (_a = shadowRoot.querySelector("a")) === null || _a === void 0 ? void 0 : _a.focus();
            }
            else if (shadowRoot.querySelector("button")) {
                (_b = shadowRoot.querySelector("button")) === null || _b === void 0 ? void 0 : _b.focus();
            }
        }
    }
    updateTheme(mode = null) {
        const foregroundColor = getBrandFromContext(this.el, mode);
        if (foregroundColor !== IcBrandForegroundEnum.Default) {
            this.monochrome = true;
            this.theme =
                foregroundColor === IcBrandForegroundEnum.Light
                    ? IcBrandForegroundEnum.Dark
                    : IcBrandForegroundEnum.Light;
        }
    }
    render() {
        const { clickable, disabled, expandable, fullWidth, heading, isFocussed, message, monochrome, href, hreflang, parentIsAnchorTag, referrerpolicy, rel, subheading, target, theme, } = this;
        const Component = parentIsAnchorTag || !clickable
            ? "div"
            : href === undefined
                ? "button"
                : "a";
        const attrs = Component == "a" && {
            href: href,
            hrefLang: hreflang,
            referrerPolicy: referrerpolicy,
            rel: rel,
            target: target,
        };
        return (h(Host, { key: '77eb6d8d39531f4eb5436fd03c38b1d4a8e4e14e', class: {
                [`ic-theme-${theme}`]: theme !== "inherit",
            } }, h(Component, Object.assign({ key: 'f602be4f06734c5fdca62b0f738b02f81988a823', class: {
                card: true,
                clickable: !!clickable && !disabled,
                disabled: !!disabled,
                fullwidth: !!fullWidth,
                focussed: isFocussed,
                monochrome: monochrome,
            }, tabindex: clickable && !parentIsAnchorTag ? 0 : undefined, "aria-disabled": disabled ? "true" : null, disabled: disabled }, attrs), isSlotUsed(this.el, "image-top") && (h("div", { key: 'e317b5bb6cac2055d7a3d0e1954e7ac77d162525', class: "image-top" }, h("slot", { key: 'e26132f7a4c806f24dce618be1a9b442b6b5caba', name: "image-top" }))), h("div", { key: '6a0efc36b72278004600f4340569c6f8dc8d11ca', class: "card-header" }, isSlotUsed(this.el, "icon") && (h("div", { key: '837470df831d655b61bc10f5b83efd02d548a017', class: "icon" }, h("slot", { key: '57a32c1e3fc6ad4739c187ebe9c8ff587c9d59f6', name: "icon" }))), h("div", { key: '06a198599be0ce2c5bda505b73fedc2d6d27f8e6', class: "card-title" }, h("slot", { key: '8e62184408b3887c340e0adb9f26b3b0a76cd287', name: "heading" }, h("ic-typography", { key: '440fe573c8ef9b176e449a10c802c1648d818337', variant: "h4" }, h("p", { key: '82ea89e26ecc579d6cdc4a153fbb25025906c018' }, heading)))), isSlotUsed(this.el, "interaction-button") && (h("div", { key: '6196db67571d24d6a2e45834e00fc44745613a10', class: "interaction-button" }, h("slot", { key: 'b8a64955d174b510f59c6425fd40c10c042a8d26', name: "interaction-button" })))), (subheading || isSlotUsed(this.el, "subheading")) && (h("div", { key: '55de98b4c457eb503ee3b6a45eaccff3dc5d9d0c', class: "subheading" }, h("slot", { key: '4560efc9da8eef089edd648f38668bb21c6ca4b6', name: "subheading" }, h("ic-typography", { key: 'b24a33d8d94f2c910a64d866f4da1664f6ff10a3', variant: "subtitle-small" }, subheading)))), isSlotUsed(this.el, "adornment") && (h("div", { key: '6766128835ebd7944b228ef4a70b47af7aa46658', class: "adornment" }, h("slot", { key: '9aa586884ad3a69c936a9ff7c831f13afb63696e', name: "adornment" }))), isSlotUsed(this.el, "image-mid") && (h("div", { key: 'e0b686295c0b455b1a33bb9dba1d16d973101b6c', class: "image-mid" }, h("slot", { key: '581dada577f03ff0c5c17d9e55d69124b71fbe96', name: "image-mid" }))), (message || isSlotUsed(this.el, "message")) && (h("div", { key: '26616626b87d6c5a36705aba4de57b8ec9b68c88', class: "card-message" }, message && (h("ic-typography", { key: 'bc14188c6005720fe4412ba375b7044f851bbcb3', variant: "body" }, message)), isSlotUsed(this.el, "message") && h("slot", { key: '6d5a43f06888b83703c33b41555676ccad3a8aaa', name: "message" }))), (isSlotUsed(this.el, "interaction-controls") || expandable) && (h("div", { key: '7896f819a4caf027eb0e883875b4a4a1ee4174da', class: "interaction-area" }, h("div", { key: '4109dcf04ac202f9e8ba1568c96e1acd7c7675fb', class: "interaction-controls" }, h("slot", { key: '42f5ba00458637177e62621852ecad1e974dd209', name: "interaction-controls" })), expandable && (h("ic-tooltip", { key: 'd4049e70edb1f6618d2f394e4f5c4f78a38b2cef', id: "ic-tooltip-expand-button", label: "Toggle expandable area", silent: true }, h("button", { key: 'eb6363669ab6002ffe50fb8ea81fca63790781fb', class: {
                "toggle-button": true,
                [`toggle-button-${this.areaExpanded ? "expanded" : "closed"}`]: true,
            }, "aria-label": "Toggle expandable area", "aria-expanded": `${this.areaExpanded}`, "aria-controls": this.areaExpanded ? "expanded-content-area" : null, onClick: this.toggleExpanded, innerHTML: chevronIcon }))))), isSlotUsed(this.el, "expanded-content") && this.areaExpanded && (h("div", { key: '54cf44b01090436c67a2dfa3b1265e924a3142b8', class: "expanded-content", id: "expanded-content-area" }, h("slot", { key: '5cc85bf2a225f6a6f1d7436f9086a5d43bf43ddf', name: "expanded-content" }))), isSlotUsed(this.el, "badge") && h("slot", { key: '20734165707440fc4ed806a179c0b7c249cd1dcf', name: "badge" }))));
    }
    static get is() { return "ic-card-vertical"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() {
        return {
            "$": ["ic-card-vertical.css"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["ic-card-vertical.css"]
        };
    }
    static get properties() {
        return {
            "clickable": {
                "type": "boolean",
                "mutable": true,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "If `true`, the card will be a clickable variant, instead of static."
                },
                "getter": false,
                "setter": false,
                "attribute": "clickable",
                "reflect": false,
                "defaultValue": "false"
            },
            "disabled": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "If `true`, the card will be disabled if it is clickable."
                },
                "getter": false,
                "setter": false,
                "attribute": "disabled",
                "reflect": false,
                "defaultValue": "false"
            },
            "expandable": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "If `true`, the card will have an expandable area and expansion toggle button."
                },
                "getter": false,
                "setter": false,
                "attribute": "expandable",
                "reflect": false,
                "defaultValue": "false"
            },
            "fullWidth": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "If `true`, the card will fill the width of the container."
                },
                "getter": false,
                "setter": false,
                "attribute": "full-width",
                "reflect": false,
                "defaultValue": "false"
            },
            "heading": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The heading for the card. This is required, unless a slotted heading is used."
                },
                "getter": false,
                "setter": false,
                "attribute": "heading",
                "reflect": false
            },
            "href": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The URL that the clickable card link points to. If set, the clickable card will render as an \"a\" tag, otherwise it will render as a button."
                },
                "getter": false,
                "setter": false,
                "attribute": "href",
                "reflect": false
            },
            "hreflang": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The human language of the linked URL."
                },
                "getter": false,
                "setter": false,
                "attribute": "hreflang",
                "reflect": false,
                "defaultValue": "\"\""
            },
            "message": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The main body message of the card."
                },
                "getter": false,
                "setter": false,
                "attribute": "message",
                "reflect": false,
                "defaultValue": "\"\""
            },
            "referrerpolicy": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "ReferrerPolicy",
                    "resolved": "\"\" | \"no-referrer\" | \"no-referrer-when-downgrade\" | \"origin\" | \"origin-when-cross-origin\" | \"same-origin\" | \"strict-origin\" | \"strict-origin-when-cross-origin\" | \"unsafe-url\" | undefined",
                    "references": {
                        "ReferrerPolicy": {
                            "location": "global",
                            "id": "global::ReferrerPolicy"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "How much of the referrer to send when following the link."
                },
                "getter": false,
                "setter": false,
                "attribute": "referrerpolicy",
                "reflect": false
            },
            "rel": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The relationship of the linked URL as space-separated link types."
                },
                "getter": false,
                "setter": false,
                "attribute": "rel",
                "reflect": false
            },
            "subheading": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The subheading for the card."
                },
                "getter": false,
                "setter": false,
                "attribute": "subheading",
                "reflect": false
            },
            "target": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The place to display the linked URL, as the name for a browsing context (a tab, window, or iframe)."
                },
                "getter": false,
                "setter": false,
                "attribute": "target",
                "reflect": false
            },
            "theme": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "IcThemeMode",
                    "resolved": "\"dark\" | \"inherit\" | \"light\" | undefined",
                    "references": {
                        "IcThemeMode": {
                            "location": "import",
                            "path": "../../utils/types",
                            "id": "src/utils/types.ts::IcThemeMode"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Sets the theme color to the dark or light theme color. \"inherit\" will set the color based on the system settings or ic-theme component."
                },
                "getter": false,
                "setter": false,
                "attribute": "theme",
                "reflect": false,
                "defaultValue": "\"inherit\""
            }
        };
    }
    static get states() {
        return {
            "areaExpanded": {},
            "isFocussed": {},
            "monochrome": {},
            "parentEl": {},
            "parentIsAnchorTag": {}
        };
    }
    static get methods() {
        return {
            "setFocus": {
                "complexType": {
                    "signature": "() => Promise<void>",
                    "parameters": [],
                    "references": {
                        "Promise": {
                            "location": "global",
                            "id": "global::Promise"
                        }
                    },
                    "return": "Promise<void>"
                },
                "docs": {
                    "text": "Sets focus on the card.",
                    "tags": []
                }
            }
        };
    }
    static get elementRef() { return "el"; }
    static get watchers() {
        return [{
                "propName": "disabled",
                "methodName": "watchDisabledHandler"
            }];
    }
    static get listeners() {
        return [{
                "name": "click",
                "method": "handleHostClick",
                "target": undefined,
                "capture": true,
                "passive": false
            }, {
                "name": "brandChange",
                "method": "brandChangeHandler",
                "target": "document",
                "capture": false,
                "passive": false
            }];
    }
}
//# sourceMappingURL=ic-card-vertical.js.map
