import { Host, h, } from "@stencil/core";
import { isSlotUsed, onComponentRequiredPropUndefined, renderHiddenInput, removeHiddenInput, addFormResetListener, removeFormResetListener, removeDisabledFalse, isPropDefined, } from "../../utils/helpers";
/**
 * @slot additional-field - Content to be displayed alongside a checkbox.
 */
// eslint-disable-next-line @typescript-eslint/no-unused-vars
export class Checkbox {
    constructor() {
        /**
         * The style of additionalField that will be displayed if used.
         */
        this.additionalFieldDisplay = "static";
        /**
         * If `true`, the checkbox will be set to the checked state. This is only the initial state and will be updated to unchecked if the checkbox is clicked.
         */
        this.checked = false;
        this.initiallyChecked = this.checked;
        /**
         * If `true`, the checkbox will be set to the disabled state.
         */
        this.disabled = false;
        /**
         * The text to be displayed when dynamic.
         */
        this.dynamicText = "This selection requires additional answers";
        /**
         * If `true`, the indeterminate state will be displayed when checked.
         */
        this.indeterminate = false;
        this.displayIndeterminate = this.indeterminate;
        /**
         * If `true`, the checkbox will behave like a native checkbox where the `indeterminate` prop sets the indeterminate visual styling, independent of the `checked` state.
         */
        this.nativeIndeterminateBehaviour = false;
        /**
         * Sets the theme color to the dark or light theme color. "inherit" will set the color based on the system settings or ic-theme component.
         */
        this.theme = "inherit";
        /**
         * If `true`, the label will be hidden and the required label value will be applied as an aria-label.
         */
        this.hideLabel = false;
        this.handleClick = () => {
            this.checked = !this.checked;
            this.displayIndeterminate = this.nativeIndeterminateBehaviour
                ? false
                : this.indeterminate && this.checked;
            this.icCheck.emit();
        };
        this.handleFormReset = () => {
            this.checked = this.initiallyChecked;
        };
    }
    watchDisabledHandler() {
        removeDisabledFalse(this.disabled, this.el);
    }
    watchIndeterminateHandler() {
        this.displayIndeterminate = this.nativeIndeterminateBehaviour
            ? this.indeterminate
            : this.indeterminate && !!this.checked;
    }
    componentWillLoad() {
        removeDisabledFalse(this.disabled, this.el);
        addFormResetListener(this.el, this.handleFormReset);
        const checkboxGroup = this.el.parentElement;
        if (checkboxGroup) {
            if (!this.name)
                this.name = checkboxGroup.name;
            this.groupLabel = checkboxGroup.label;
        }
    }
    componentDidLoad() {
        onComponentRequiredPropUndefined([
            { prop: this.label, propName: "label" },
            { prop: this.value, propName: "value" },
        ], "Checkbox");
    }
    componentDidRender() {
        if (this.additionalFieldDisplay === "static") {
            const textfield = this.el.querySelector("ic-text-field");
            if (!this.checked) {
                textfield === null || textfield === void 0 ? void 0 : textfield.setAttribute("disabled", "");
            }
            else {
                textfield === null || textfield === void 0 ? void 0 : textfield.removeAttribute("disabled");
            }
        }
        else if (this.additionalFieldContainer) {
            this.additionalFieldContainer.style.display = !this.checked
                ? "none"
                : "flex";
        }
    }
    disconnectedCallback() {
        removeFormResetListener(this.el, this.handleFormReset);
    }
    /**
     * Sets focus on the checkbox.
     */
    async setFocus() {
        var _a, _b;
        (_b = (_a = this.el.shadowRoot) === null || _a === void 0 ? void 0 : _a.querySelector(".checkbox")) === null || _b === void 0 ? void 0 : _b.focus();
    }
    render() {
        const { additionalFieldDisplay, checked, disabled, dynamicText, el, form, displayIndeterminate, groupLabel, label, name, size, value, theme, } = this;
        const id = `ic-checkbox-${isPropDefined(label) || value}-${groupLabel}`.replace(/ /g, "-");
        const parentElementSize = el.parentElement
            .size;
        checked
            ? renderHiddenInput(true, el, name, checked && value, disabled)
            : removeHiddenInput(el);
        return (h(Host, { key: '1b4f1fd6e5e8848ef63134541cbc314fe5976b2b', class: {
                "ic-checkbox-disabled": !!disabled,
                [`ic-checkbox-${size || parentElementSize}`]: true,
                [`ic-theme-${theme}`]: theme !== "inherit",
            } }, h("div", { key: '3ec8b6cc45dcf275c6340c2d7321e335c9383ac5', class: "container" }, displayIndeterminate && h("div", { key: 'd38202c75ef2f3a90a8b57efcb4030fc222e1038', class: "indeterminate-symbol" }), !displayIndeterminate && checked && (h("svg", { key: '134ea738eb2858a4a27ed8792edb7668790d59a7', class: "checkmark", viewBox: "0 0 24 24", xmlns: "http://www.w3.org/2000/svg", "fill-rule": "evenodd", "clip-rule": "evenodd" }, h("title", { key: 'e8113684e52bb9e6b66200886a8aef0e4eeeaf28' }, "checkmark icon"), h("path", { key: '342f98d89977388cfabb77fdf08cb4ecb7d7cc14', d: "M21 6.285l-11.16 12.733-6.84-6.018 1.319-1.49 5.341 4.686 9.865-11.196 1.475 1.285z" }))), h("input", { key: '1b0ad31864fc8d7b7d40bc50e975162cf387ae01', role: "checkbox", class: {
                checkbox: true,
                checked: !!checked,
                indeterminate: !!displayIndeterminate,
            }, type: "checkbox", name: name, id: id, value: value, disabled: disabled, checked: checked, indeterminate: displayIndeterminate, onClick: this.handleClick, form: form, "aria-label": this.hideLabel ? this.label : undefined }), !this.hideLabel && (h("ic-typography", { key: '4e0aacf944928ed74162bb779ea2aa7f69cfd88d', class: "checkbox-label", variant: "body" }, h("label", { key: 'cd6a349ecf42e327806161ff4eb38680a4b9d91b', htmlFor: id }, label)))), isSlotUsed(el, "additional-field") && (h("div", { key: 'd658a44ea59a92e254ec4d5e097b43205c81d06a', class: "dynamic-container", ref: (el) => (this.additionalFieldContainer = el) }, additionalFieldDisplay === "dynamic" && (h("div", { key: 'ed31c233420f6f8c88951e7d208c7bcc418d382a', class: "branch-corner" })), h("div", { key: '9cffdb6170b03336348940da135cb8c686235610' }, additionalFieldDisplay === "dynamic" && (h("ic-typography", { key: 'c6b1518d6f7641afaba0befc8de0a8ccb23cc7e9', variant: "caption" }, h("p", { key: '931a9b34320375e0ff79014aa9ae4e9100a3c860', class: "dynamic-text", "aria-live": "polite" }, dynamicText))), h("div", { key: '1d1034308c125fd0ec86a5bcfe91103742141b8c', class: {
                "additional-field-wrapper": additionalFieldDisplay === "static",
            } }, h("slot", { key: '6b3c352f0fb38399cfdea480687fc69d39358e7a', name: "additional-field" })))))));
    }
    static get is() { return "ic-checkbox"; }
    static get encapsulation() { return "shadow"; }
    static get delegatesFocus() { return true; }
    static get originalStyleUrls() {
        return {
            "$": ["ic-checkbox.css"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["ic-checkbox.css"]
        };
    }
    static get properties() {
        return {
            "additionalFieldDisplay": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "IcAdditionalFieldTypes",
                    "resolved": "\"dynamic\" | \"static\" | undefined",
                    "references": {
                        "IcAdditionalFieldTypes": {
                            "location": "import",
                            "path": "../../utils/types",
                            "id": "src/utils/types.ts::IcAdditionalFieldTypes"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The style of additionalField that will be displayed if used."
                },
                "getter": false,
                "setter": false,
                "attribute": "additional-field-display",
                "reflect": true,
                "defaultValue": "\"static\""
            },
            "checked": {
                "type": "boolean",
                "mutable": true,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "If `true`, the checkbox will be set to the checked state. This is only the initial state and will be updated to unchecked if the checkbox is clicked."
                },
                "getter": false,
                "setter": false,
                "attribute": "checked",
                "reflect": true,
                "defaultValue": "false"
            },
            "disabled": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "If `true`, the checkbox will be set to the disabled state."
                },
                "getter": false,
                "setter": false,
                "attribute": "disabled",
                "reflect": false,
                "defaultValue": "false"
            },
            "dynamicText": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The text to be displayed when dynamic."
                },
                "getter": false,
                "setter": false,
                "attribute": "dynamic-text",
                "reflect": false,
                "defaultValue": "\"This selection requires additional answers\""
            },
            "form": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The <form> element to associate the checkbox with."
                },
                "getter": false,
                "setter": false,
                "attribute": "form",
                "reflect": false
            },
            "groupLabel": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The group label for the checkbox."
                },
                "getter": false,
                "setter": false,
                "attribute": "group-label",
                "reflect": false
            },
            "indeterminate": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "If `true`, the indeterminate state will be displayed when checked."
                },
                "getter": false,
                "setter": false,
                "attribute": "indeterminate",
                "reflect": false,
                "defaultValue": "false"
            },
            "label": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": true,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "The label for the checkbox."
                },
                "getter": false,
                "setter": false,
                "attribute": "label",
                "reflect": false
            },
            "name": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The name for the checkbox. If not set when used in a checkbox group, the name will be based on the group name."
                },
                "getter": false,
                "setter": false,
                "attribute": "name",
                "reflect": false
            },
            "nativeIndeterminateBehaviour": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "If `true`, the checkbox will behave like a native checkbox where the `indeterminate` prop sets the indeterminate visual styling, independent of the `checked` state."
                },
                "getter": false,
                "setter": false,
                "attribute": "native-indeterminate-behaviour",
                "reflect": false,
                "defaultValue": "false"
            },
            "size": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "IcSizes",
                    "resolved": "\"large\" | \"medium\" | \"small\" | undefined",
                    "references": {
                        "IcSizes": {
                            "location": "import",
                            "path": "../../utils/types",
                            "id": "src/utils/types.ts::IcSizes"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The size of the checkbox to be displayed. This does not affect the font size of the label. If a checkbox is contained in a checkbox group, this will override the size set on checkbox group."
                },
                "getter": false,
                "setter": false,
                "attribute": "size",
                "reflect": false
            },
            "theme": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "IcThemeMode",
                    "resolved": "\"dark\" | \"inherit\" | \"light\" | undefined",
                    "references": {
                        "IcThemeMode": {
                            "location": "import",
                            "path": "../../utils/types",
                            "id": "src/utils/types.ts::IcThemeMode"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Sets the theme color to the dark or light theme color. \"inherit\" will set the color based on the system settings or ic-theme component."
                },
                "getter": false,
                "setter": false,
                "attribute": "theme",
                "reflect": false,
                "defaultValue": "\"inherit\""
            },
            "value": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": true,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "The value for the checkbox."
                },
                "getter": false,
                "setter": false,
                "attribute": "value",
                "reflect": false
            },
            "hideLabel": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "If `true`, the label will be hidden and the required label value will be applied as an aria-label."
                },
                "getter": false,
                "setter": false,
                "attribute": "hide-label",
                "reflect": false,
                "defaultValue": "false"
            }
        };
    }
    static get states() {
        return {
            "initiallyChecked": {},
            "displayIndeterminate": {}
        };
    }
    static get events() {
        return [{
                "method": "icCheck",
                "name": "icCheck",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "Emitted when a checkbox has been checked."
                },
                "complexType": {
                    "original": "void",
                    "resolved": "void",
                    "references": {}
                }
            }];
    }
    static get methods() {
        return {
            "setFocus": {
                "complexType": {
                    "signature": "() => Promise<void>",
                    "parameters": [],
                    "references": {
                        "Promise": {
                            "location": "global",
                            "id": "global::Promise"
                        },
                        "HTMLElement": {
                            "location": "global",
                            "id": "global::HTMLElement"
                        }
                    },
                    "return": "Promise<void>"
                },
                "docs": {
                    "text": "Sets focus on the checkbox.",
                    "tags": []
                }
            }
        };
    }
    static get elementRef() { return "el"; }
    static get watchers() {
        return [{
                "propName": "disabled",
                "methodName": "watchDisabledHandler"
            }, {
                "propName": "indeterminate",
                "methodName": "watchIndeterminateHandler"
            }];
    }
}
//# sourceMappingURL=ic-checkbox.js.map
