import { Host, h, } from "@stencil/core";
import { IcBrandForegroundEnum, } from "../../utils/types";
import { slotHasContent, getBrandForegroundAppearance, onComponentRequiredPropUndefined, isPropDefined, isSlotUsed, renderDynamicChildSlots, } from "../../utils/helpers";
/**
 * @slot heading - Content will be rendered in the title area, in place of the heading.
 * @slot subheading - Content will be rendered in the title area, in place of the subheading.
 * @slot interaction - Content will be rendered in the interaction area, adjacent to the title area.
 * @slot secondary - Content will be rendered in the secondary content area.
 */
export class Hero {
    constructor() {
        this.hostMutationObserver = null;
        this.foregroundColor = getBrandForegroundAppearance();
        this.rightContent = false;
        // set by above state
        this.leftContentFullWidth = !this.rightContent && this.secondaryHeading === undefined;
        this.scrollFactor = "right -100px";
        /**
         * The alignment of the hero.
         */
        this.aligned = "left";
        /**
         * The alignment of the hero content.
         */
        this.contentAligned = "left";
        /**
         * If `true`, the background image (if set) will not scroll using a parallax effect.
         */
        this.disableBackgroundParallax = false;
        /**
         * The size of the hero component.
         */
        this.size = "medium";
    }
    disconnectedCallback() {
        var _a;
        (_a = this.hostMutationObserver) === null || _a === void 0 ? void 0 : _a.disconnect();
    }
    componentWillLoad() {
        this.rightContent = slotHasContent(this.el, "secondary");
    }
    componentDidLoad() {
        !isSlotUsed(this.el, "heading") &&
            onComponentRequiredPropUndefined([{ prop: this.heading, propName: "heading" }], "Hero");
        this.hostMutationObserver = new MutationObserver((mutationList) => renderDynamicChildSlots(mutationList, "secondary", this));
        this.hostMutationObserver.observe(this.el, {
            childList: true,
        });
    }
    componentWillRender() {
        this.rightContent = slotHasContent(this.el, "secondary");
    }
    brandChangeHandler(ev) {
        this.foregroundColor = ev.detail.mode;
    }
    doScroll() {
        var _a;
        if (this.disableBackgroundParallax) {
            return;
        }
        const scrolltotop = (_a = document.scrollingElement) === null || _a === void 0 ? void 0 : _a.scrollTop;
        if (scrolltotop) {
            const factor = 0.4;
            const y = -100 + scrolltotop * factor;
            this.scrollFactor = "right " + y + "px";
        }
    }
    render() {
        const { aligned, size, heading, subheading, secondaryHeading, secondarySubheading, foregroundColor, backgroundImage, scrollFactor, } = this;
        let style = {};
        if (isPropDefined(backgroundImage)) {
            style = {
                "background-image": "url(" + backgroundImage + ")",
                "background-position": scrollFactor,
            };
        }
        return (h(Host, { key: '74dcaf107650373274e01fe04fd1964c3bb62073', class: {
                [`ic-hero-${IcBrandForegroundEnum.Dark}`]: foregroundColor === IcBrandForegroundEnum.Dark,
                ["has-background-image"]: backgroundImage !== undefined,
                ["ic-hero-small"]: size === "small",
                ["secondary-heading"]: !!secondaryHeading,
            }, style: style }, h("ic-section-container", { key: '18df0272497747b6660942829636b2d3225913e9', aligned: aligned, fullHeight: true, class: "section-container" }, h("div", { key: '6ecd484adbcb987f3f90ff58ee6f749c9a3422ed', class: {
                ["left-container"]: true,
                ["left-container-full-width"]: this.leftContentFullWidth,
            } }, h("div", { key: '8d4c4b0a35417046b06a2dbfd1e13e40183bd7fb', class: "heading" }, h("slot", { key: 'fc5aa8fd7b8dcb255d3a9405444b17670e3e9d7b', name: "heading" }, h("ic-typography", { key: '360e0e4ff5b9c44a707011abcb33771bb711226b', variant: size === "small" ? "h2" : "h1", class: {
                ["heading-bottom-spacing"]: size !== "small",
            } }, h("h2", { key: '671b42db7002c2d77d42f8eb087af469910ac4ba' }, heading)))), h("div", { key: 'd967221ce6e889da8a749f7c3deacbed0fb70211', class: "subheading" }, h("slot", { key: '3363059a438d470e90a4782f956c838c7394fe16', name: "subheading" }, h("ic-typography", { key: 'a9e7e665d6f42f14d0b0368e3d437cacfae4edcb', variant: "body" }, subheading))), h("div", { key: 'a99a67438d1c017ed7822fcf4b46243e8af300f7', class: "interaction-container" }, h("slot", { key: '5f54b40b54fdc5696935d37dfe3ef7ae68599390', name: "interaction" }))), (isPropDefined(secondaryHeading) || this.rightContent) && (h("div", { key: '89513652cf857ead516f5b16e1a5776b135db42c', class: "right-container" }, h("slot", { key: '739e6f69d2466cf001489eea05488b98e2713f12', name: "secondary" }, secondaryHeading && (h("div", { key: '41f85c0af629eaa9fb859768b495563edf2cc734', class: "secondary-container" }, h("div", { key: '9ef7879d221cbf53e36eda0a99b59bd76bd0476f', class: "secondary-heading" }, h("ic-typography", { key: '41787caaf24919d5a7041294d73445f58aa3d724', variant: "h4" }, h("h3", { key: '1137fb4ee0e4fe64efed120d0e5ab5d4dc25bd5d' }, secondaryHeading))), h("div", { key: '2985066c33994c07e167e59ddff6d64e3713b888', class: "secondary-subheading" }, h("ic-typography", { key: '6bee21b6ce698c66dab6c49ab39558373bfec1f8', variant: "subtitle-small" }, secondarySubheading))))))))));
    }
    static get is() { return "ic-hero"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() {
        return {
            "$": ["ic-hero.css"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["ic-hero.css"]
        };
    }
    static get properties() {
        return {
            "aligned": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "IcAlignment",
                    "resolved": "\"center\" | \"full-width\" | \"left\" | undefined",
                    "references": {
                        "IcAlignment": {
                            "location": "import",
                            "path": "../../utils/types",
                            "id": "src/utils/types.ts::IcAlignment"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The alignment of the hero."
                },
                "getter": false,
                "setter": false,
                "attribute": "aligned",
                "reflect": false,
                "defaultValue": "\"left\""
            },
            "backgroundImage": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The optional background image."
                },
                "getter": false,
                "setter": false,
                "attribute": "background-image",
                "reflect": false
            },
            "contentAligned": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "IcHeroContentAlignments",
                    "resolved": "\"center\" | \"left\" | undefined",
                    "references": {
                        "IcHeroContentAlignments": {
                            "location": "import",
                            "path": "./ic-hero.types",
                            "id": "src/components/ic-hero/ic-hero.types.tsx::IcHeroContentAlignments"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The alignment of the hero content."
                },
                "getter": false,
                "setter": false,
                "attribute": "content-aligned",
                "reflect": false,
                "defaultValue": "\"left\""
            },
            "disableBackgroundParallax": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "If `true`, the background image (if set) will not scroll using a parallax effect."
                },
                "getter": false,
                "setter": false,
                "attribute": "disable-background-parallax",
                "reflect": false,
                "defaultValue": "false"
            },
            "heading": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The heading of the hero. An <h2> level heading."
                },
                "getter": false,
                "setter": false,
                "attribute": "heading",
                "reflect": false
            },
            "secondaryHeading": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The optional secondary heading, an <h3> level heading. Replaced by slotted right content."
                },
                "getter": false,
                "setter": false,
                "attribute": "secondary-heading",
                "reflect": false
            },
            "secondarySubheading": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The optional secondary subheading, replaced by slotted right content."
                },
                "getter": false,
                "setter": false,
                "attribute": "secondary-subheading",
                "reflect": false
            },
            "size": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "IcSizesNoLarge",
                    "resolved": "\"medium\" | \"small\" | undefined",
                    "references": {
                        "IcSizesNoLarge": {
                            "location": "import",
                            "path": "../../utils/types",
                            "id": "src/utils/types.ts::IcSizesNoLarge"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The size of the hero component."
                },
                "getter": false,
                "setter": false,
                "attribute": "size",
                "reflect": false,
                "defaultValue": "\"medium\""
            },
            "subheading": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The description for the hero."
                },
                "getter": false,
                "setter": false,
                "attribute": "subheading",
                "reflect": false
            }
        };
    }
    static get states() {
        return {
            "foregroundColor": {},
            "rightContent": {},
            "leftContentFullWidth": {},
            "scrollFactor": {}
        };
    }
    static get elementRef() { return "el"; }
    static get listeners() {
        return [{
                "name": "brandChange",
                "method": "brandChangeHandler",
                "target": "document",
                "capture": false,
                "passive": false
            }, {
                "name": "scroll",
                "method": "doScroll",
                "target": "document",
                "capture": false,
                "passive": true
            }];
    }
}
//# sourceMappingURL=ic-hero.js.map
