import { Host, h, } from "@stencil/core";
import { IcBrandForegroundEnum, } from "../../utils/types";
import { checkResizeObserver, DEVICE_SIZES, getCurrentDeviceSize, getBrandForegroundAppearance, getSlot, onComponentPropUndefinedChange, onComponentRequiredPropUndefined, isSlotUsed, isEmptyString, isPropDefined, } from "../../utils/helpers";
/**
 * @slot app-icon - Content will be rendered to left of app title. Anything that is slotted here will be hidden from screen readers.
 * @slot app-title - Handle routing by nesting a route in the app title.
 * @slot short-app-title - Handle routing by nesting a route in the short app title (to be displayed in place of app title on small screen sizes).
 * @slot search - Content will be rendered in search area to left of buttons.
 * @slot toggle-icon - Icon to be displayed on the button to toggle search slot content on smaller devices
 * @slot navigation - Content will be rendered in navigation panel.
 * @slot buttons - Content will be rendered to right of search bar.
 */
export class TopNavigation {
    constructor() {
        this.hasAppIcon = false;
        this.hasIconButtons = false;
        this.hasNavigation = false;
        this.hasSearchSlotContent = false;
        this.resizeObserver = null;
        this.searchBar = null;
        this.deviceSize = DEVICE_SIZES.XL;
        this.foregroundColor = getBrandForegroundAppearance();
        this.hasFullWidthSearchBar = false;
        this.mobileSearchBarVisible = false;
        this.mobileSearchHiddenOnBlur = false;
        this.navMenuVisible = false;
        this.searchButtonClick = false;
        this.searchValue = "";
        /**
         * The alignment of the top navigation content.
         */
        this.contentAligned = "full-width";
        /**
         * Can set a custom breakpoint for the top navigation to switch to mobile mode.
         * Must be one of our specified breakpoints in px: `0`, `576`, `768`, `992`, `1200`.
         */
        this.customMobileBreakpoint = DEVICE_SIZES.L;
        /**
         *  The URL to navigate to when the app title is clicked.
         */
        this.href = "/";
        /**
         * If `true`, the flyout navigation menu on small devices will be contained by the parent element.
         */
        this.inline = false;
        /**
         * The short title of the app to be displayed at small screen sizes in place of the app title.
         */
        this.shortAppTitle = "";
        /**
         * The status info to be displayed.
         */
        this.status = "";
        /**
         * Sets the theme color to the dark or light theme color. "inherit" will set the color based on the system settings or ic-theme component.
         */
        this.theme = "inherit";
        /**
         * The version info to be displayed.
         */
        this.version = "";
        this.checkSlots = () => {
            this.hasAppIcon = isSlotUsed(this.el, "app-icon");
            this.hasNavigation = isSlotUsed(this.el, "navigation");
            this.hasIconButtons = isSlotUsed(this.el, "buttons");
            this.hasSearchSlotContent = isSlotUsed(this.el, "search");
        };
        this.initialiseSearchBar = () => {
            if (this.hasSearchSlotContent) {
                const slot = getSlot(this.el, "search");
                if ((slot === null || slot === void 0 ? void 0 : slot.tagName) === "IC-SEARCH-BAR") {
                    this.searchBar = slot;
                }
                else if ((slot === null || slot === void 0 ? void 0 : slot.tagName) === "FORM") {
                    this.searchBar = slot.querySelector("ic-search-bar");
                }
                if (this.searchBar !== null) {
                    this.searchBar.hideLabel = true;
                }
            }
        };
        this.menuButtonClick = () => {
            this.showNavMenu(true);
        };
        this.searchButtonMouseDownHandler = () => {
            this.searchButtonClick = true;
        };
        this.searchButtonClickHandler = () => {
            this.toggleSearchBar();
            this.searchButtonClick = false;
        };
        this.resizeObserverCallback = (currSize) => {
            if (currSize !== this.deviceSize) {
                this.deviceSize = currSize;
                if (currSize > this.customMobileBreakpoint) {
                    this.showNavMenu(false);
                    if (this.mobileSearchBarVisible) {
                        this.toggleSearchBar();
                    }
                }
                this.icTopNavResized.emit({
                    size: currSize,
                });
                if (this.searchBar && document.activeElement === this.searchBar) {
                    this.searchBar.setAttribute("hidden", "true");
                    //remove attribute again as this trigger a redraw & applies css
                    this.searchBar.removeAttribute("hidden");
                    setTimeout(() => {
                        var _a;
                        (_a = this.searchBar) === null || _a === void 0 ? void 0 : _a.focus();
                    }, 100);
                }
            }
        };
        this.runResizeObserver = () => {
            this.resizeObserver = new ResizeObserver(() => {
                this.resizeObserverCallback(getCurrentDeviceSize());
            });
            this.resizeObserver.observe(this.el);
        };
    }
    watchPropHandler(newValue, oldValue) {
        //added for Gatsby rehydration issue where prop is initially undefined but then changes to actual value
        //this watch can be removed once Gatsby hydration issue is resolved
        onComponentPropUndefinedChange(oldValue, newValue, this.initialiseSearchBar);
    }
    disconnectedCallback() {
        var _a;
        (_a = this.resizeObserver) === null || _a === void 0 ? void 0 : _a.disconnect();
    }
    componentWillLoad() {
        this.checkSlots();
        this.deviceSize = getCurrentDeviceSize();
        this.initialiseSearchBar();
    }
    componentDidLoad() {
        checkResizeObserver(this.runResizeObserver);
        !isSlotUsed(this.el, "app-title") &&
            onComponentRequiredPropUndefined([{ prop: this.appTitle, propName: "app-title" }], "Top Navigation");
    }
    componentWillRender() {
        this.checkSlots();
    }
    navBarMenuCloseHandler() {
        var _a, _b;
        this.showNavMenu(false);
        (_b = (_a = this.el.shadowRoot) === null || _a === void 0 ? void 0 : _a.querySelector("#menu-button")) === null || _b === void 0 ? void 0 : _b.focus();
    }
    searchInputBlurHandler({ detail, }) {
        if (detail !== null) {
            if (this.mobileSearchBarVisible && !this.searchButtonClick) {
                //don't hide if blur was triggered by click on search button - let the click handler toggle the state
                this.toggleSearchBar();
            }
            this.searchValue = detail.value;
        }
    }
    searchValueChangeHandler({ detail }) {
        this.searchValue = detail.value;
    }
    brandChangeHandler({ detail }) {
        this.foregroundColor = detail.mode;
    }
    toggleSearchBar() {
        var _a;
        this.mobileSearchBarVisible = !this.mobileSearchBarVisible;
        if (this.searchBar !== null) {
            (_a = this.mobileSearchButtonEl) === null || _a === void 0 ? void 0 : _a.setAttribute("aria-label", `${this.mobileSearchBarVisible ? "Hide" : "Show"} search`);
            this.hasFullWidthSearchBar = this.mobileSearchBarVisible;
            this.searchBar.fullWidth = this.mobileSearchBarVisible;
            if (this.mobileSearchBarVisible) {
                setTimeout(() => {
                    var _a;
                    (_a = this.searchBar) === null || _a === void 0 ? void 0 : _a.focus();
                }, 100);
            }
        }
    }
    showNavMenu(show) {
        this.navMenuVisible = show;
        (show ? this.icNavigationMenuOpened : this.icNavigationMenuClosed).emit();
        document.body.style.height = show ? "100%" : "auto";
        document.body.style.overflow = show ? "hidden" : "auto";
    }
    render() {
        const { appTitle, contentAligned, customMobileBreakpoint, deviceSize, el, foregroundColor, hasAppIcon, hasFullWidthSearchBar, hasIconButtons, hasNavigation, hasSearchSlotContent, href, inline, menuButtonClick, mobileSearchBarVisible, navMenuVisible, searchButtonClickHandler, searchButtonMouseDownHandler, shortAppTitle, status, version, theme, } = this;
        const hasStatus = status !== "";
        const hasVersion = version !== "";
        const hasMenuContent = hasNavigation || hasIconButtons || hasStatus || hasVersion;
        const searchButtonSize = deviceSize <= DEVICE_SIZES.S ? "medium" : "large";
        const hasTitle = appTitle !== "" && isPropDefined(appTitle);
        const overMobileBreakpoint = deviceSize <= customMobileBreakpoint;
        const appTitleVariant = overMobileBreakpoint
            ? deviceSize <= DEVICE_SIZES.S
                ? "subtitle-small"
                : "h4"
            : "h3";
        const mobileSearchButtonTitle = `${mobileSearchBarVisible ? "Hide" : "Show"} search`;
        const menuSize = deviceSize <= DEVICE_SIZES.S ? "small" : "medium";
        const shortAppTitleSlot = isSlotUsed(el, "short-app-title");
        const hasAppTitleSlot = isSlotUsed(el, "app-title");
        const Component = hasAppTitleSlot ? "div" : "a";
        const attrs = Component == "a" && {
            href: href,
        };
        return (h(Host, { key: '5c5bfc3ad297f7bd59b148652ccc478b896b6008', class: {
                "fullwidth-searchbar": hasFullWidthSearchBar,
                "mobile-mode": overMobileBreakpoint,
                [IcBrandForegroundEnum.Dark]: foregroundColor === IcBrandForegroundEnum.Dark,
                [`ic-theme-${theme}`]: theme !== "inherit",
            } }, h("div", { key: '7b20f493f5738050c11ad8447113b7322f46c3c4', class: "top-navigation" }, h("ic-section-container", { key: 'abce0c4c81da4ed68094058d623e8ed954224fa0', aligned: contentAligned, "full-height": true }, h("header", { key: '30860fda7f50065dd28399e6630836a6ca66f9fa', role: "banner" }, h("div", { key: '6341b2155fc652ae76ea2266612fa56add3b2a87', class: "top-panel-container" }, h("div", { key: '70b84e9110139cdc1bccdc5fc79c60983966e7e2', class: "app-details-container" }, (hasTitle || hasAppTitleSlot) && (h(Component, Object.assign({ key: 'b54b79365cfd322a778aa1de952eda4daee04520', class: "title-link" }, attrs), hasAppIcon && (h("div", { key: 'a390fa98b041b07a02a33cb787a0902ed369effa', class: "app-icon-container" }, h("slot", { key: 'c33e9295f6d196d8823b59f4def9779dbf8be3d9', name: "app-icon" }))), deviceSize <= DEVICE_SIZES.S &&
            (!isEmptyString(shortAppTitle) || shortAppTitleSlot) ? (h("ic-typography", { variant: "subtitle-small", "aria-label": (!hasAppTitleSlot || !shortAppTitleSlot) &&
                `${appTitle} (${shortAppTitle})` }, h("h1", null, shortAppTitleSlot ? (h("slot", { name: "short-app-title" })) : (shortAppTitle)))) : (h("ic-typography", { variant: appTitleVariant }, h("h1", { class: "title-wrap" }, hasAppTitleSlot ? (h("slot", { name: "app-title" })) : (appTitle)))))), hasStatus && (h("div", { key: 'f240cb39a4c6456695745f676d263fd5fc7c5b8c', class: "app-status" }, h("ic-typography", { key: 'f8ae131fb476805ec7e1cfe88c4fba9c7a436c10', "aria-label": "app tag", variant: "label-uppercase", class: "app-status-text" }, status))), hasVersion && (h("div", { key: '1cdf5652e0b56c70162a884f45e648fce2093ea5', class: "app-version" }, h("ic-typography", { key: '2444bbf26e72433f1bd1cd4c3986de8cadea85b6', variant: "label", class: "app-version-text", "aria-label": "app version" }, version)))), (hasSearchSlotContent || hasMenuContent) && (h("div", { key: '54fac6a943b3701376fc3f7a308f2b6c586aa636', class: "search-menu-container" }, h("div", { key: '5ef5f824b967f0b7f8802d2fdbd77056ae8f1129', class: "search-actions-container" }, !overMobileBreakpoint && h("slot", { key: '16551964dd909010294a0e6a59763f95a5d0f728', name: "search" }), hasSearchSlotContent && overMobileBreakpoint && (h("ic-button", { key: 'c8a339bdb6389bd43e973c20707a13f7fa0a4e1e', id: "search-toggle-button", ref: (el) => (this.mobileSearchButtonEl = el), onMouseDown: searchButtonMouseDownHandler, variant: "icon-tertiary", monochrome: true, size: searchButtonSize, "aria-label": mobileSearchButtonTitle, theme: foregroundColor, onClick: searchButtonClickHandler }, h("slot", { key: '66f8326868c0cb23cd989df5de784f2eb2630319', name: "toggle-icon" }, h("svg", { key: '2c4e0354d201371de737a0d3c72455e543b3205a', xmlns: "http://www.w3.org/2000/svg", viewBox: "0 0 24 24", fill: "#ffffff" }, h("path", { key: 'e1adbe066080455b1b801c711b025318dab908eb', d: "M0 0h24v24H0V0z", fill: "none" }), h("path", { key: 'dadeb94826ed263b91566b46cc753748351db453', d: "M15.5 14h-.79l-.28-.27C15.41 12.59 16 11.11 16 9.5 16 5.91 13.09 3 9.5 3S3 5.91 3 9.5 5.91 16 9.5 16c1.61 0 3.09-.59 4.23-1.57l.27.28v.79l5 4.99L20.49 19l-4.99-5zm-6 0C7.01 14 5 11.99 5 9.5S7.01 5 9.5 5 14 7.01 14 9.5 11.99 14 9.5 14z" }))))), hasIconButtons && !overMobileBreakpoint && (h("div", { key: '36eedc891ad8653ffa1a812306e60f76c8e82ed8', class: "icon-buttons-container" }, h("slot", { key: '1061472fc71131df7541a0654e4096ff39ad5e3d', name: "buttons" }))), hasMenuContent && overMobileBreakpoint && (h("div", { key: '1b7503905b2528ca36e3f57b58c9e08958e1ab60', class: "menu-button-container" }, h("span", { key: '93d4150b9660243082f5d1ebb40fe33450d7785b', id: "navigation-landmark-button-text", class: "navigation-landmark-button-text", "aria-hidden": "true" }, "Main navigation button"), h("nav", { key: 'a356ce42d0d4865dfce33a49d768c73dcd824e3c', "aria-labelledby": "navigation-landmark-button-text", "aria-hidden": `${navMenuVisible}` }, h("ic-button", { key: 'd2f0fe6637e181fcfd2dbb5c33de93cdd2a1fb32', id: "menu-button", theme: foregroundColor, variant: "secondary", monochrome: true, "aria-expanded": "false", "aria-haspopup": "true", "aria-label": `Open ${hasNavigation ? "navigation" : "app"} menu`, size: menuSize, onClick: menuButtonClick }, "Menu", h("svg", { key: 'cd8e405c4cd82075ec2f8b192f01a5f268ffd520', xmlns: "http://www.w3.org/2000/svg", height: "24px", viewBox: "0 0 24 24", width: "24px", fill: "#ffffff", slot: "left-icon" }, h("path", { key: '1a91d6099a80507845ae6165931b457dcc416d6a', d: "M0 0h24v24H0V0z", fill: "none" }), h("path", { key: '87dc16c578d8c379a00766769dd0959a70855166', d: "M3 18h18v-2H3v2zm0-5h18v-2H3v2zm0-7v2h18V6H3z" })))))))))), mobileSearchBarVisible && (h("div", { key: '088982083ccb21a658b8ba378e532f280d6d563a', class: "search-bar-container" }, h("slot", { key: 'ad3e3c4c632d3e5431bcf6ebe954c45d3c7e6233', name: "search" }))), hasNavigation && !overMobileBreakpoint && (h("div", { key: '6aeb6f3e8bb5d1474c2c7ede10c49d44cec31545', class: "navigation-tabs" }, h("span", { key: '8a965acd6056d216c9da63173012250fa72be222', id: "navigation-landmark-text", class: "navigation-landmark-text", "aria-hidden": "true" }, "Main pages"), h("nav", { key: '7b36045fbc1efc8d0f471a67cb8bccb6cdb7ca62', "aria-labelledby": "navigation-landmark-text", class: "nav-panel-container" }, h("ic-horizontal-scroll", { key: 'ac328ac5ddd86614f3d727ddc236cbf6dee019b9', monochrome: true, appearance: foregroundColor }, h("ul", { key: '7324ed8f489a813b706e18e7db14d8f8201ee4fe', class: "navigation-item-list", tabindex: "-1" }, h("slot", { key: '7266abcda953294233cc4c4525ba5c47f61a95bb', name: "navigation" }))))))))), navMenuVisible && (h("ic-navigation-menu", { key: '0fff5404ab1ec895f011dc1643d0d2b51b5eac33', version: version, status: status, class: {
                ["inline"]: !!inline,
            } }, hasIconButtons && (h("div", { key: 'a7b45891338fb80858cead62559e1de75505ec96', class: "menu-buttons-slot", slot: "buttons" }, h("slot", { key: '09d6e44a68a6368ae5388a6101aedd75fcbecb2a', name: "buttons" }))), h("ul", { key: 'ff2e7168dacb64ede053528ce21c1211de506fd7', slot: "navigation" }, h("slot", { key: '218681289b2170fd04553d1e4b8da7f1ae72b448', name: "navigation" }))))));
    }
    static get is() { return "ic-top-navigation"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() {
        return {
            "$": ["ic-top-navigation.css"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["ic-top-navigation.css"]
        };
    }
    static get properties() {
        return {
            "contentAligned": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "IcAlignment",
                    "resolved": "\"center\" | \"full-width\" | \"left\" | undefined",
                    "references": {
                        "IcAlignment": {
                            "location": "import",
                            "path": "../../utils/types",
                            "id": "src/utils/types.ts::IcAlignment"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The alignment of the top navigation content."
                },
                "getter": false,
                "setter": false,
                "attribute": "content-aligned",
                "reflect": false,
                "defaultValue": "\"full-width\""
            },
            "customMobileBreakpoint": {
                "type": "number",
                "mutable": false,
                "complexType": {
                    "original": "IcDeviceSizes",
                    "resolved": "0 | 1200 | 576 | 768 | 992 | undefined",
                    "references": {
                        "IcDeviceSizes": {
                            "location": "import",
                            "path": "../../utils/types",
                            "id": "src/utils/types.ts::IcDeviceSizes"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Can set a custom breakpoint for the top navigation to switch to mobile mode.\nMust be one of our specified breakpoints in px: `0`, `576`, `768`, `992`, `1200`."
                },
                "getter": false,
                "setter": false,
                "attribute": "custom-mobile-breakpoint",
                "reflect": false,
                "defaultValue": "DEVICE_SIZES.L"
            },
            "href": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The URL to navigate to when the app title is clicked."
                },
                "getter": false,
                "setter": false,
                "attribute": "href",
                "reflect": false,
                "defaultValue": "\"/\""
            },
            "inline": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "If `true`, the flyout navigation menu on small devices will be contained by the parent element."
                },
                "getter": false,
                "setter": false,
                "attribute": "inline",
                "reflect": false,
                "defaultValue": "false"
            },
            "shortAppTitle": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The short title of the app to be displayed at small screen sizes in place of the app title."
                },
                "getter": false,
                "setter": false,
                "attribute": "short-app-title",
                "reflect": false,
                "defaultValue": "\"\""
            },
            "status": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The status info to be displayed."
                },
                "getter": false,
                "setter": false,
                "attribute": "status",
                "reflect": false,
                "defaultValue": "\"\""
            },
            "theme": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "IcThemeMode",
                    "resolved": "\"dark\" | \"inherit\" | \"light\" | undefined",
                    "references": {
                        "IcThemeMode": {
                            "location": "import",
                            "path": "../../utils/types",
                            "id": "src/utils/types.ts::IcThemeMode"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Sets the theme color to the dark or light theme color. \"inherit\" will set the color based on the system settings or ic-theme component."
                },
                "getter": false,
                "setter": false,
                "attribute": "theme",
                "reflect": false,
                "defaultValue": "\"inherit\""
            },
            "version": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The version info to be displayed."
                },
                "getter": false,
                "setter": false,
                "attribute": "version",
                "reflect": false,
                "defaultValue": "\"\""
            },
            "appTitle": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The app title to be displayed. This is required, unless a slotted app title link is used."
                },
                "getter": false,
                "setter": false,
                "attribute": "app-title",
                "reflect": false
            }
        };
    }
    static get states() {
        return {
            "deviceSize": {},
            "foregroundColor": {},
            "hasFullWidthSearchBar": {},
            "mobileSearchBarVisible": {},
            "mobileSearchHiddenOnBlur": {},
            "navMenuVisible": {},
            "searchButtonClick": {},
            "searchValue": {}
        };
    }
    static get events() {
        return [{
                "method": "icNavigationMenuClosed",
                "name": "icNavigationMenuClosed",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [{
                            "name": "internal",
                            "text": "- Emitted when the menu is closed."
                        }],
                    "text": ""
                },
                "complexType": {
                    "original": "void",
                    "resolved": "void",
                    "references": {}
                }
            }, {
                "method": "icNavigationMenuOpened",
                "name": "icNavigationMenuOpened",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [{
                            "name": "internal",
                            "text": "- Emitted when the menu is opened."
                        }],
                    "text": ""
                },
                "complexType": {
                    "original": "void",
                    "resolved": "void",
                    "references": {}
                }
            }, {
                "method": "icTopNavResized",
                "name": "icTopNavResized",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "Emitted when the top navigation is resized."
                },
                "complexType": {
                    "original": "{ size: number }",
                    "resolved": "{ size: number; }",
                    "references": {}
                }
            }];
    }
    static get elementRef() { return "el"; }
    static get watchers() {
        return [{
                "propName": "appTitle",
                "methodName": "watchPropHandler"
            }];
    }
    static get listeners() {
        return [{
                "name": "icNavigationMenuClose",
                "method": "navBarMenuCloseHandler",
                "target": undefined,
                "capture": false,
                "passive": false
            }, {
                "name": "icSearchBarBlur",
                "method": "searchInputBlurHandler",
                "target": undefined,
                "capture": false,
                "passive": false
            }, {
                "name": "icChange",
                "method": "searchValueChangeHandler",
                "target": undefined,
                "capture": false,
                "passive": false
            }, {
                "name": "brandChange",
                "method": "brandChangeHandler",
                "target": "document",
                "capture": false,
                "passive": false
            }];
    }
}
//# sourceMappingURL=ic-top-navigation.js.map
